@@ -52,14 +52,14 @@ void SQLiteDatabase::Connect () throw (Xception)
 
 		m_pSession = new Session(SQLite::Connector::KEY, m_connect);
 	}
-	catch (const SQLite::CantOpenDBFileException& )
+	catch (const SQLite::CantOpenDBFileException& x)
 	{
 		if (m_pSession)
 		{
 			delete m_pSession;
 			m_pSession = NULL;
 		}
-		throw Xception(format("cannot open database: %s") % m_connect);
+		throw Xception(format("cannot open database: %s, %s") % m_connect % x.displayText());
 	}
 
 	// check presence of a schema, create schema if neccessary
@@ -72,7 +72,7 @@ void SQLiteDatabase::Connect () throw (Xception)
 		(*m_pSession) << "select * from TCollections;", now;
 		has_schema = true;
 	}
-	catch (const SQLite::InvalidSQLStatementException& )
+	catch (...)
 	{
 		// no schema!
 	}
@@ -82,21 +82,43 @@ void SQLiteDatabase::Connect () throw (Xception)
 		if (m_verbose)
 			cout << "creating new schema" << endl;
 
-		m_pSession->begin();
 		try
 		{
-			(*m_pSession) << "create table TCollections (name varchar(256) not null primary key, type char(1) not null, tbl_name varchar(256) not null);", now;
-			(*m_pSession) << "create table TInserted (t char(19) not null, expr varchar(1024) not null, constraint PK_TInserted primary key(t, expr));", now;
-			(*m_pSession) << "create table TRejected (t char(19) not null, expr varchar(1024) not null, constraint PK_TRejected primary key(t, expr));", now;
-			(*m_pSession) << "create table C0 (t char(19) not null primary key, v double not null);", now;
-			(*m_pSession) << "create table C1 (t char(19) not null primary key, v varchar(1024) not null);", now;
-			(*m_pSession) << "insert into TCollections values ('*', 'n', 'C0');", now;
-			(*m_pSession) << "insert into TCollections values ('#', 't', 'C1');", now;
-			m_pSession->commit();
+			DBTransactor transactor(m_pSession);
+
+			(*m_pSession) << "create table TCollections "
+					 "("
+						"name varchar(64) not null primary key,"
+						"type char(1) not null,"
+						"tbl_name varchar(64) not null"
+					 ");", now;
+
+			(*m_pSession) << "create table TRejected "
+					 "("
+						"i integer primary key autoincrement,"
+						"t char(19) not null,"
+						"expr varchar(1024) not null"
+					 ");", now;
+
+			(*m_pSession) << "create table C0 "
+					 "("
+						"t char(19) not null primary key,"
+						"v double not null"
+					 ");", now;
+
+			(*m_pSession) << "create table C1 "
+					 "("
+						"t char(19) not null primary key,"
+						"v varchar(1024) not null"
+					 ");", now;
+
+			(*m_pSession) << "insert into TCollections values ('*','n','C0');", now;
+			(*m_pSession) << "insert into TCollections values ('#','t','C1');", now;
+
+			transactor.Commit();
 		}
 		catch (...)
 		{
-			m_pSession->rollback();
 			throw Xception("could not create schema");
 		}
 	}
@@ -127,21 +149,21 @@ void SQLiteDatabase::ListCollections () throw (Xception)
 		}
 		string operator () (int idx)
 		{
-			static const size_t widths[] = {6, 8, 6, 7, 20}; // echte Spaltenbreiten
+			static const size_t widths[] = {6, 8, 6, 7, 20}; // real column widths
 			size_t width = widths[idx];
 			string result(m_s);
 			while (result.length() < width)
 				result += ' ';
 			if (result.length() > width)
 				result.erase(width - 1);
-			return result + ' '; // + 1 Leerzeichen als Spaltentrenner
+			return result + ' '; // + 1 space as column delimiter
 		}
 	};
 
 	try
 	{
 		Statement select(*m_pSession);
-		select << "select name, type, tbl_name from TCollections order by name;";
+		select << "select name, type,tbl_name from TCollections order by name;";
 		select.execute();
 
 		RecordSet rs(select);
@@ -163,13 +185,13 @@ void SQLiteDatabase::ListCollections () throw (Xception)
 
 				int count;
 				string min, max;
-				(*m_pSession) << "select count(*), min(t), max(t) from " << rs.value(2).convert<string>() << ";", into(count), into(min), into(max), now;
+				(*m_pSession) << "select count(*),min(t),max(t) from " << rs.value(2).convert<string>() << ";", into(count), into(min), into(max), now;
 				cout << fill(lexical_cast<string>(count))(3) << fill(min)(4) << max << endl;
 
 			} while (rs.moveNext());
 		}
 	}
-	catch (const Poco::NotFoundException& )
+	catch (...)
 	{
 	}
 }
@@ -179,56 +201,58 @@ void SQLiteDatabase::AddCollection (const string& name) throw (Xception) // "nam
 	if (m_verbose)
 		cout << "adding collection" << endl;
 
-	// split collection name and type
-	regex rx("\\s*([^,\\s]+)(?:\\s*,\\s*([nrt])\\s*)?");
-	smatch mr;
-	if (!regex_match(name, mr, rx))
-		throw Xception(format("illegal collection name: %s") % name);
-	string n(mr[1]);
-	string t(mr[2]);
-	if (t.empty())
-		t = "n"; // numeric
-
-	// check for built-in collection
-	if (n == "*" || n == "#")
-		throw Xception(format("cannot create built-in collection: %s") % n);
-
-	// check for existing collection
-	int count = 0;
-	(*m_pSession) << "select count(*) from TCollections where name = :name;", use(n), into(count), now;
-	if (count != 0)
-		throw Xception(format("collection to add already exists: %s") % n);
-
-	// build new table name
-	string tbl_name;
-	(*m_pSession) << "select max(tbl_name) from TCollections", into(tbl_name), now;
-	tbl_name.erase(0, 1); // C
-	int i = lexical_cast<int>(tbl_name);
-	tbl_name = (format("C%d") % ++i).str();
-
-	// create collection
-	m_pSession->begin();
 	try
 	{
-		string sql("create table ");
-		sql += tbl_name;
-		sql += " (t char(19) not null primary key, ";
-		switch (t[0])
-		{
-			case 'n':	sql += "v double not null"; break;
-			case 'r':	sql += "n double not null, d double not null"; break;
-			case 't':	sql += "v varchar(1024) not null"; break;
-		}
-		sql += ");";
-		(*m_pSession) << sql, now;
-		(*m_pSession) << "insert into TCollections values (:n, :t, :tbl_name)", use(n), use(t), use(tbl_name), now;
+		// split collection name and type
+		regex rx("\\s*([^,\\s]+)(?:\\s*,\\s*([nrt])\\s*)?");
+		smatch mr;
+		if (!regex_match(name, mr, rx))
+			throw Xception(format("illegal collection name: %s") % name);
+		string n(mr[1]);
+		string t(mr[2]);
+		if (t.empty())
+			t = "n"; // numeric
 
-		m_pSession->commit();
+		// check for built-in collection
+		if (n == "*" || n == "#")
+			throw Xception(format("cannot create built-in collection: %s") % n);
+
+		// check for existing collection
+		int count = 0;
+		(*m_pSession) << "select count(*) from TCollections where name=:name;", use(n), into(count), now;
+		if (count != 0)
+			throw Xception(format("collection already exists: %s") % n);
+
+		// build new table name
+		string tbl_name;
+		(*m_pSession) << "select max(tbl_name) from TCollections;", into(tbl_name), now;
+		tbl_name.erase(0, 1); // C
+		int i = lexical_cast<int>(tbl_name);
+		tbl_name = (format("C%d") % ++i).str();
+
+		{
+			// create collection
+			DBTransactor transactor(m_pSession);
+
+			string sql("create table ");
+			sql += tbl_name;
+			sql += " (t char(19) not null primary key,";
+			switch (t[0])
+			{
+				case 'n':	sql += "v double not null"; break;
+				case 'r':	sql += "n double not null,d double not null"; break;
+				case 't':	sql += "v varchar(1024) not null"; break;
+			}
+			sql += ");";
+			(*m_pSession) << sql, now;
+			(*m_pSession) << "insert into TCollections values (:n,:t,:tbl_name);", use(n), use(t), use(tbl_name), now;
+
+			transactor.Commit();
+		}
 	}
 	catch (...)
 	{
-		m_pSession->rollback();
-		throw Xception(format("could not create collection: %s") % n);
+		throw Xception(format("could not create collection: %s") % name);
 	}
 }
 
@@ -237,30 +261,32 @@ void SQLiteDatabase::DeleteCollection (const string& name) throw (Xception)
 	if (m_verbose)
 		cout << "deleting collection" << endl;
 
-	// check for built-in collection
-	if (name == "*" || name == "#")
-		throw Xception(format("cannot delete built-in collection: %s") % name);
-
-	// check for unknown collection
-	int count = 0;
-	(*m_pSession) << "select count(*) from TCollections where name = :name;", use(name), into(count), now;
-	if (count != 1)
-		throw Xception(format("unknown collection to delete: %s") % name);
-
-	// delete collection
-	m_pSession->begin();
 	try
 	{
-		string tbl_name;
-		(*m_pSession) << "select tbl_name from TCollections where name = :name;", use(name), into(tbl_name), now;
-		(*m_pSession) << "delete from TCollections where name = :name;", use(name), now;
-		(*m_pSession) << "drop table " << tbl_name << ";", now;
+		// check for built-in collection
+		if (name == "*" || name == "#")
+			throw Xception(format("cannot delete built-in collection: %s") % name);
 
-		m_pSession->commit();
+		// check for unknown collection
+		int count = 0;
+		(*m_pSession) << "select count(*) from TCollections where name=:name;", use(name), into(count), now;
+		if (count != 1)
+			throw Xception(format("unknown collection: %s") % name);
+
+		{
+			// delete collection
+			DBTransactor transactor(m_pSession);
+
+			string tbl_name;
+			(*m_pSession) << "select tbl_name from TCollections where name=:name;", use(name), into(tbl_name), now;
+			(*m_pSession) << "delete from TCollections where name = :name;", use(name), now;
+			(*m_pSession) << "drop table " << tbl_name << ";", now;
+
+			transactor.Commit();
+		}
 	}
 	catch (...)
 	{
-		m_pSession->rollback();
 		throw Xception(format("could not delete collection: %s") % name);
 	}
 }
@@ -270,12 +296,13 @@ void SQLiteDatabase::DeleteAllCollections () throw (Xception)
 	if (m_verbose)
 		cout << "deleting all collections" << endl;
 
-	m_pSession->begin();
 	try
 	{
+		DBTransactor transactor(m_pSession);
+
 		{
 			Statement select(*m_pSession);
-			select << "select name, tbl_name from TCollections where name not in ('*', '#');";
+			select << "select name,tbl_name from TCollections where name not in ('*','#');";
 			select.execute();
 
 			RecordSet rs(select);
@@ -287,15 +314,14 @@ void SQLiteDatabase::DeleteAllCollections () throw (Xception)
 			}
 		}
 
-		(*m_pSession) << "delete from TCollections where name not in ('*', '#');", now;
+		(*m_pSession) << "delete from TCollections where name not in ('*','#');", now;
 		(*m_pSession) << "delete from C0;", now;
 		(*m_pSession) << "delete from C1;", now;
 
-		m_pSession->commit();
+		transactor.Commit();
 	}
 	catch (...)
 	{
-		m_pSession->rollback();
 		throw Xception("could not delete collections");
 	}
 }
@@ -309,12 +335,12 @@ char SQLiteDatabase::GetCollectionType (const string& name) throw (Xception)
 			return (*I).second;
 
 		string type;
-		(*m_pSession) << "select type from TCollections where name='" << name << "';", into(type), now;
+		(*m_pSession) << "select type from TCollections where name=:name;", use(name), into(type), now;
 		m_collectionTypes.insert(CollectionTypes::value_type(name, type[0]));
 
 		return type[0];
 	}
-	catch (const Poco::NotFoundException& )
+	catch (...)
 	{
 		throw Xception(format("unknown collection: %s") % name);
 	}
@@ -333,14 +359,14 @@ void SQLiteDatabase::ListRejections () throw (Xception)
 		}
 		string operator () (int idx)
 		{
-			static const size_t widths[] = {20}; // echte Spaltenbreiten
+			static const size_t widths[] = {20}; // real column widths
 			size_t width = widths[idx];
 			string result(m_s);
 			while (result.length() < width)
 				result += ' ';
 			if (result.length() > width)
 				result.erase(width - 1);
-			return result + ' '; // + 1 Leerzeichen als Spaltentrenner
+			return result + ' '; // + 1 space as column delimiter
 		}
 	};
 
@@ -348,7 +374,7 @@ void SQLiteDatabase::ListRejections () throw (Xception)
 	{
 
 		Statement select(*m_pSession);
-		select << "select t, expr from TRejected order by t;";
+		select << "select t,expr from TRejected order by t;";
 		select.execute();
 
 		RecordSet rs(select);
@@ -360,7 +386,7 @@ void SQLiteDatabase::ListRejections () throw (Xception)
 			} while (rs.moveNext());
 		}
 	}
-	catch (const Poco::NotFoundException& e)
+	catch (...)
 	{
 	}
 }
@@ -382,14 +408,15 @@ void SQLiteDatabase::DeleteAllRejections () throw (Xception)
 
 void SQLiteDatabase::AddCollectionElements (const CollectionElements& elements) throw (Xception)
 {
-	m_pSession->begin();
 	try
 	{
+		DBTransactor transactor(m_pSession);
+
 		foreach (const CollectionElement& element, elements)
 		{
 			// get collection type and table name
 			string type, tbl_name;
-			(*m_pSession) << "select type, tbl_name from TCollections where name=:name;", use(element.m_name), into(type), into(tbl_name), now;
+			(*m_pSession) << "select type,tbl_name from TCollections where name=:name;", use(element.m_name), into(type), into(tbl_name), now;
 
 			// insert or update?
 			const string& t = lexical_cast<string, ptime>(element.m_timestamp);
@@ -401,7 +428,7 @@ void SQLiteDatabase::AddCollectionElements (const CollectionElements& elements)
 			{
 				double data = any_cast<double>(element.m_data);
 				if (count == 0)
-					(*m_pSession) << "insert into " << tbl_name << " values (:t, :v);", use(t), use(data), now;
+					(*m_pSession) << "insert into " << tbl_name << " values (:t,:v);", use(t), use(data), now;
 				else
 					(*m_pSession) << "update " << tbl_name << " set v=:v where t=:t;", use(data), use(t), now;
 			}
@@ -411,9 +438,9 @@ void SQLiteDatabase::AddCollectionElements (const CollectionElements& elements)
 				{
 					Ratio data = any_cast<Ratio>(element.m_data);
 					if (count == 0)
-						(*m_pSession) << "insert into " << tbl_name << " values (:t, :n, :d);", use(t), use(data.m_numerator), use(data.m_denominator), now;
+						(*m_pSession) << "insert into " << tbl_name << " values (:t,:n,:d);", use(t), use(data.m_numerator), use(data.m_denominator), now;
 					else
-						(*m_pSession) << "update " << tbl_name << " set n=:n, d=:d where t=:t;", use(data.m_numerator), use(data.m_denominator), use(t), now;
+						(*m_pSession) << "update " << tbl_name << " set n=:n,d=:d where t=:t;", use(data.m_numerator), use(data.m_denominator), use(t), now;
 				}
 				else
 				{
@@ -421,32 +448,19 @@ void SQLiteDatabase::AddCollectionElements (const CollectionElements& elements)
 					{
 						string data = any_cast<string>(element.m_data);
 						if (count == 0)
-							(*m_pSession) << "insert into " << tbl_name << " values (:t, :v);", use(t), use(data), now;
+							(*m_pSession) << "insert into " << tbl_name << " values (:t,:v);", use(t), use(data), now;
 						else
 							(*m_pSession) << "update " << tbl_name << " set v=:v where t=:t;", use(data), use(t), now;
 					}
 				}
 			}
 		}
-		m_pSession->commit();
-	}
-	catch (...)
-	{
-		m_pSession->rollback();
-		throw Xception("could not insert or update values");
-	}
-}
 
-void SQLiteDatabase::AddInserted (const ptime& timestamp, const string& expr) throw (Xception)
-{
-	try
-	{
-		const string& t = lexical_cast<string, ptime>(timestamp);
-		(*m_pSession) << "insert into TInserted values (:timestamp, :expr);", use(t), use(expr), now;
+		transactor.Commit();
 	}
 	catch (...)
 	{
-		throw Xception("could not insert expression");
+		throw Xception("could not insert or update values");
 	}
 }
 
@@ -455,7 +469,7 @@ void SQLiteDatabase::AddRejected (const ptime& timestamp, const string& expr) th
 	try
 	{
 		const string& t = lexical_cast<string, ptime>(timestamp);
-		(*m_pSession) << "insert into TRejected values (:timestamp, :expr);", use(t), use(expr), now;
+		(*m_pSession) << "insert into TRejected values (null,:timestamp,:expr);", use(t), use(expr), now;
 	}
 	catch (...)
 	{
