@@ -24,8 +24,8 @@ using namespace boost::program_options;
 
 #include "../libpdrx/datatypes.h"
 #include "../libpdrx/config.h"
+#include "../libpdrx/build.h"
 #include "db.h"
-#include "build.h"
 #include "out_ftree.h"
 #include "diagram.h"
 
@@ -2308,6 +2308,12 @@ DEFINE_FIMPL_0(string_build, typeid(string), "build")
 	return any(string(BUILD));
 }
 
+// (database)
+DEFINE_FIMPL_0(string_database, typeid(string), "database")
+{
+	return any(context.m_database.GetVersion());
+}
+
 //--- diagram -------------------------------------------------------------
 
 // (nothing diagram width height bg_color ...)
@@ -2660,6 +2666,7 @@ FImplFactory::FImplFactory ()
 	m_impls.insert(new estring_empty_string());
 	m_impls.insert(new string_version());
 	m_impls.insert(new string_build());
+	m_impls.insert(new string_database());
 
 	// diagram functions
 	m_impls.insert(new nothing_diagram_int_int_color_ellipse());
